/*
    Mastermind 1.3
    Copyright (C) 2004 Christian Helbling
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


// C Source File
// Created 03.12.2003; 19:32:27


#include <tigcclib.h>         // Include All Header Files

#include "extgraph.h"
#include "mstrmnd.h"


int select (int x, int y1, int ystep, int maxsel, int *sel) {

	int key=0, done=0, y=y1+ystep*( *sel-1);

	while (!done) {
				
		key=ngetchx();
		
		switch (key)
		{
			case KEY_DOWN:
			
			Sprite8_XOR(x,y,8,flash,LCD_MEM);
			
			y+=ystep;
			(*sel)++;
			
			if (*sel>maxsel) {
				*sel=1;
				y=y1;
			}

			Sprite8_XOR(x,y,8,flash,LCD_MEM);
		
			break;


			case KEY_UP:

			Sprite8_XOR(x,y,8,flash,LCD_MEM);
			
			y-=ystep;
			(*sel)--;
			
			if (*sel<1) {
				*sel=maxsel;
				y=y1+(maxsel-1)*ystep;
			}
			Sprite8_XOR(x,y,8,flash,LCD_MEM);
			break;
			
			case KEY_ENTER:
			case KEY_RIGHT:
			case KEY_ESC:
			case KEY_F5:
			case KEY_LEFT:
			done=1;
			break;

		}
		
	}
	
	return key;
}


void showHighscores (HSC *highscores, int *options) {
	
	int a;
	char buffer[3];

	border();
	
	FontSetSys(F_4x6);

	DrawStr(2,20,"HIGHSCORES:",A_NORMAL);
	
	if (!options[2]) {
		DrawStr(50,20,"multiple symbols disabled:",A_NORMAL);
		DrawStr(8,30,"symbols:",A_NORMAL);
		DrawStr(48,30,"name:",A_NORMAL);
		DrawStr(98,30,"trials:",A_NORMAL);		
	} else {
		DrawStr(50,20,"mehrfache Symbole aus:",A_NORMAL);
		DrawStr(8,30,"Symbole:",A_NORMAL);
		DrawStr(48,30,"Name:",A_NORMAL);
		DrawStr(92,30,"Versuche:",A_NORMAL);				
	}


	DrawStr(8,32,"______________________________",A_NORMAL);
	for (a=0; a<7; a++) {
		sprintf(buffer,"%d",a+4);
		DrawStr(16,39+a*7,buffer,A_NORMAL);
		DrawStr(48,39+a*7,highscores[a].name,A_NORMAL);	
		sprintf(buffer,"%d",highscores[a].score);
		DrawStr(100,39+a*7,buffer,A_NORMAL);
	}
	DrawStr(8,83,"______________________________",A_NORMAL);
	
	FontSetSys(F_6x8);

	anyKey(options);
	
	
	ClrScr();
	
	
	border();
	
	FontSetSys(F_4x6);

	DrawStr(2,20,"HIGHSCORES:",A_NORMAL);
	
	if (!options[2]) {
		DrawStr(50,20,"multiple symbols enabled:",A_NORMAL);
		DrawStr(8,30,"symbols:",A_NORMAL);
		DrawStr(48,30,"name:",A_NORMAL);
		DrawStr(98,30,"trials:",A_NORMAL);		
	} else {
		DrawStr(50,20,"mehrfache Symbole an:",A_NORMAL);
		DrawStr(8,30,"Symbole:",A_NORMAL);
		DrawStr(48,30,"Name:",A_NORMAL);
		DrawStr(92,30,"Versuche:",A_NORMAL);				
	}


	DrawStr(8,32,"_____________________________",A_NORMAL);
	for (a=0; a<7; a++) {
		sprintf(buffer,"%d",a+4);
		DrawStr(16,39+a*7,buffer,A_NORMAL);
		DrawStr(48,39+a*7,highscores[a+7].name,A_NORMAL);	
		sprintf(buffer,"%d",highscores[a+7].score);
		DrawStr(100,39+a*7,buffer,A_NORMAL);
	}
	DrawStr(8,83,"_____________________________",A_NORMAL);
	
	FontSetSys(F_6x8);

	anyKey(options);




}



// Custom String Input Function
void InputStr(char *buffer, unsigned short maxlen, int x, int y)
{
  short key;
  unsigned short i = 0;
  buffer[0] = 0;
  char dispstr[maxlen+3];
	do
    {
 	    sprintf(dispstr,"%s_  ",buffer);
      DrawStr(x,y,dispstr,A_REPLACE);
      key = ngetchx ();
      if (key >= ' ' && key <= '~' && i < maxlen)
        buffer[i++] = key;
      else if (key == KEY_BACKSPACE && i)
        i--;
      buffer[i] = 0;
    } while (key != KEY_ENTER || i<3);
}


void readFile(int *options, HSC *highscores) {
	FILE *fsav;
	if ((fsav = fopen(SAVEFILE,"rb")) != NULL) {
		
		fread((void *)options,sizeof(int),3,fsav);
		fread((void *)highscores,sizeof(HSC),14,fsav);
			
		fclose(fsav);		

	} else {
		options[0] = 0;
		options[1] = 8;
		options[2] = 0;
		int a;
		for (a=0; a<14; a++) {
			highscores[a].score = 100;
			sprintf(highscores[a].name," --- ");
		}
		saveFile(options,highscores);
	}
}

void saveFile(int *options, HSC *highscores) {
	FILE *fsav;
	
	if ((fsav = fopen(SAVEFILE,"wb")) != NULL) {
	
		fwrite((void *)options,sizeof(int),3,fsav);
		fwrite((void *)highscores,sizeof(HSC),14,fsav);
		
		fputc(0,fsav);
		fputs("mstr",fsav);
		fputc(0,fsav);
		fputc(OTH_TAG,fsav);
		
		fclose(fsav);
	}
}


int newGame(int trials[NR_OF_HISTORY_ENTRIES][8], int *options, HSC *highscores) {
	

	
	int solution[4];
	int temparray[NR_OF_SYMBOLS];
	int temp,randnumber;
	int a,b,key=0,done=0;
	int currentSymbol=0;
	int cpx=0;
	int cpy=0;
	int invisibleleft=0;
	int attempt=0;
	
	unsigned char *pxaddr;
	char sAttempt[4];
	char sCpx[4];
	
	for (a=0; a<NR_OF_HISTORY_ENTRIES; a++) {
	for (b=0; b<8; b++) {
				trials[a][b]=0;
	}
	}
	
	
	ClrScr();
	
	
	if (!options[0]) {
		
		//easy version:
		for (a=0; a<options[1]; a++) {
			temparray[a] = a;
		}
		for (a=0; a<options[1]; a++) {
			randnumber = random(options[1]) ;
			temp = temparray[randnumber];
			temparray[randnumber] = temparray[a];
			temparray[a] = temp;
	
		}
		for (a=0; a<4; a++) {
			solution[a] = temparray[a];
		}
		
	} else {
		
		//difficult version:	
		for (a=0; a<NR_OF_SYMBOLS; a++) {
			solution[a] = random(options[1]);
		}
		
	}


	border();

	Sprite8_XOR(4,37,8,question_mark,LCD_MEM);
	Sprite8_XOR(4,49,8,question_mark,LCD_MEM);
	Sprite8_XOR(4,61,8,question_mark,LCD_MEM);
	Sprite8_XOR(4,73,8,question_mark,LCD_MEM);
	
	Sprite16_XOR(3,36,10,cursor2,LCD_MEM);
	Sprite16_XOR(3,48,10,cursor2,LCD_MEM);
	Sprite16_XOR(3,60,10,cursor2,LCD_MEM);
	Sprite16_XOR(3,72,10,cursor2,LCD_MEM);
	
	FastDrawLine(LCD_MEM,14,19,14,99,A_NORMAL);
	FastDrawLine(LCD_MEM,0,84,159,84,A_NORMAL);
	FastDrawLine(LCD_MEM,0,33,159,33,A_NORMAL);
	
	Sprite8_XOR(23+12*cpx,37,8,symbols[0],LCD_MEM);
	Sprite8_XOR(23+12*cpx,49,8,symbols[0],LCD_MEM);
	Sprite8_XOR(23+12*cpx,61,8,symbols[0],LCD_MEM);
	Sprite8_XOR(23+12*cpx,73,8,symbols[0],LCD_MEM);

	
	Sprite16_XOR(21+12*cpx,35+12*cpy,12,cursor1,LCD_MEM);
	
	while (!done) {
		
		FastDrawLine(LCD_MEM,0,99,159,99,A_NORMAL);
		
		sprintf(sCpx,"%2d ",cpx+1);
		sprintf(sAttempt,"%2d ",attempt+1);
		FontSetSys(F_4x6);
		DrawStr(3,24,sCpx,A_REPLACE);
		DrawStr(3,89,sAttempt,A_REPLACE);
		FontSetSys(F_6x8);
		
		
		if (invisibleleft>0) {
			Sprite8_OR(16,55,8,scrollLeft,LCD_MEM);
		} else {
			for (pxaddr = EXT_PIXADDR(LCD_MEM,16,55); pxaddr < EXT_PIXADDR(LCD_MEM,16,63); pxaddr+=30) {
				*pxaddr = ((*pxaddr) & 0x0F);			
			}
		}
	
		if (attempt-invisibleleft>10) {
			Sprite8_OR(154,55,8,scrollRight,LCD_MEM);
		} else {
			for (pxaddr = EXT_PIXADDR(LCD_MEM,154,55); pxaddr < EXT_PIXADDR(LCD_MEM,154,63); pxaddr+=30) {
				*pxaddr = ((*pxaddr) & 0x83);			
			}
		}
			
		key=ngetchx();
		
		switch (key) {
			case KEY_ESC:
			done =1;
			break;

			case KEY_CLEAR:			
			case KEY_F2:
			if (attempt == cpx) {
				trials[cpx][cpy]=changeSymbol(currentSymbol, options);
				Sprite8_XOR(23+12*(cpx-invisibleleft),37+cpy*12,8,symbols[currentSymbol],LCD_MEM);
				currentSymbol = trials[cpx][cpy];
				Sprite8_XOR(23+12*(cpx-invisibleleft),37+cpy*12,8,symbols[currentSymbol],LCD_MEM);
					
				attempt = cpx;
			}
			break;


			case KEY_DOWN:
			Sprite16_XOR(21+12*(cpx-invisibleleft),35+12*cpy,12,cursor1,LCD_MEM);
			cpy++;
			if (cpy>3) {
				cpy=0;
			}
			Sprite16_XOR(21+12*(cpx-invisibleleft),35+12*cpy,12,cursor1,LCD_MEM);
			currentSymbol=trials[cpx][cpy];
			break;


			case KEY_UP:
			Sprite16_XOR(21+12*(cpx-invisibleleft),35+12*cpy,12,cursor1,LCD_MEM);
			cpy--;
			if (cpy<0) {
				cpy=3;
			}
			Sprite16_XOR(21+12*(cpx-invisibleleft),35+12*cpy,12,cursor1,LCD_MEM);			
			currentSymbol=trials[cpx][cpy];
			break;
					
	
			case KEY_LEFT:
			Sprite16_XOR(21+12*(cpx-invisibleleft),35+12*cpy,12,cursor1,LCD_MEM);
			cpx--;
			if (cpx<0) {
				cpx=0;
			}
			if (cpx==invisibleleft-1 && invisibleleft!=0) {
				invisibleleft--;
				
				//scroll 12px to the right:
				scrollRight12();
				
				for (a=0; a<4; a++) {
					Sprite8_XOR(23+12*(cpx-invisibleleft),37+a*12,8,symbols[trials[cpx][a]],LCD_MEM);
				}
				drawHint( trials, cpx, invisibleleft);
			}
			
			Sprite16_XOR(21+12*(cpx-invisibleleft),35+12*cpy,12,cursor1,LCD_MEM);			
			currentSymbol=trials[cpx][cpy];
			break;



			case KEY_RIGHT:
			Sprite16_XOR(21+12*(cpx-invisibleleft),35+12*cpy,12,cursor1,LCD_MEM);
			cpx++;
			if (cpx>attempt) {
				cpx=attempt;
			}
			
			if (cpx==11+invisibleleft) {
				invisibleleft++;
				
				//scroll 12px to the right:
				scrollLeft12();
				
				for (a=0; a<4; a++) {
					Sprite8_XOR(23+12*(cpx-invisibleleft),37+a*12,8,symbols[trials[cpx][a]],LCD_MEM);
				}
			}
			
			if (cpx!=attempt) {
				drawHint(trials,cpx,invisibleleft);
			}
			
			Sprite16_XOR(21+12*(cpx-invisibleleft),35+12*cpy,12,cursor1,LCD_MEM);			
			currentSymbol=trials[cpx][cpy];
			break;
			
			
			case KEY_F1:
			case KEY_ENTER:
			if (cpx==NR_OF_HISTORY_ENTRIES-2) {
					if (!options[2]) {
						DrawStr(20,22,"Sorry, no more trials!",A_NORMAL);
					} else {
						DrawStr(20,22,"Keine Versuche mehr!",A_NORMAL);						
					}
					ngetchx();
					done=1;
					return 0;
			}
			done = try(trials, solution, &attempt,&cpx, &cpy, &invisibleleft, &currentSymbol, options, highscores);			
			break;

			case KEY_F5:						//exit program
				return 1;
			break; 
				
		}
	
	}
	return 0;
}


int editOptions (int *options, HSC *highscores) {
	int sel=1, key=0, done=0, exit=0;
	char sOp[5];

	border();
	
	if (!options[2]) {
		DrawStr(22,40,"Multiple symbols:  ",A_REPLACE);
		DrawStr(22,50,"Number of symbols: ",A_REPLACE);
		DrawStr(22,60,"Language:          ",A_REPLACE);
		DrawStr(22,70,"Return to main     ",A_REPLACE);
	} else {
		DrawStr(22,40,"Mehrfache Symbole: ",A_REPLACE);
		DrawStr(22,50,"Anzahl Symbole:    ",A_REPLACE);
		DrawStr(22,60,"Sprache:           ",A_REPLACE);
		DrawStr(22,70,"Zurck zum Men    ",A_REPLACE);		
	}

  	Sprite8_XOR(12,30+10*sel,8,flash,LCD_MEM);

	
	while (!done) {	

		FontSetSys(F_4x6);

		if (!options[0]) {
			if (!options[2]) {
				DrawStr(126,41,"disabled ",A_REPLACE);
			} else {
				DrawStr(132,41,"aus      ",A_REPLACE);				
			}
		} else {
			if (!options[2]) {
				DrawStr(126,41,"enabled  ",A_REPLACE);
			} else {
				DrawStr(132,41,"an       ",A_REPLACE);				
			}

		}

		if (!options[2]) {
			DrawStr(78,61,"English  ",A_REPLACE);
		} else {
			DrawStr(74,61,"Deutsch   ",A_REPLACE);
		}


		if (!options[2]) {	
			sprintf(sOp,"%d ",options[1]);
		} else {
			sprintf(sOp,"%d    ",options[1]);			
		}
			
		FontSetSys(F_6x8);
		if (!options[2]) {
			DrawStr(134,50,sOp,A_REPLACE);		
		} else {
			DrawStr(120,50,sOp,A_REPLACE);					
		}


		key = select (12, 40, 10, 4, &sel);
		
		switch (key) {
			case KEY_RIGHT:
			case KEY_ENTER:
			
			switch (sel) {
	
				case 1:
				options[0] = !options[0];
				break;
				
				case 2:
				options[1]++;
				if (options[1] == 11) {
					options[1] = 4;
				}
				break;
				
				case 3:
				options[2] = !options[2];
				if (!options[2]) {
					DrawStr(22,40,"Multiple symbols:  ",A_REPLACE);
					DrawStr(22,50,"Number of symbols: ",A_REPLACE);
					DrawStr(22,60,"Language:          ",A_REPLACE);
					DrawStr(22,70,"Return to main     ",A_REPLACE);
				} else {
					DrawStr(22,40,"Mehrfache Symbole: ",A_REPLACE);
					DrawStr(22,50,"Anzahl Symbole:    ",A_REPLACE);
					DrawStr(22,60,"Sprache:           ",A_REPLACE);
					DrawStr(22,70,"Zurck zum Men    ",A_REPLACE);		
				}
				break;
							
				case 4:
				saveFile(options,highscores);
				done = 1;
				break;
			}
				
			break;
			
			case KEY_LEFT:

			switch (sel) {
	
				case 1:
				options[0] = !options[0];
				break;
				
				case 2:
				options[1]--;
				if (options[1] == 3) {
					options[1] = 10;
				}
				break;
				
				case 3:
				options[2] = !options[2];
				if (!options[2]) {
					DrawStr(22,40,"Multiple symbols:  ",A_REPLACE);
					DrawStr(22,50,"Number of symbols: ",A_REPLACE);
					DrawStr(22,60,"Language:          ",A_REPLACE);
					DrawStr(22,70,"Return to main     ",A_REPLACE);
				} else {
					DrawStr(22,40,"Mehrfache Symbole: ",A_REPLACE);
					DrawStr(22,50,"Anzahl Symbole:    ",A_REPLACE);
					DrawStr(22,60,"Sprache:           ",A_REPLACE);
					DrawStr(22,70,"Zurck zum Men    ",A_REPLACE);		
				}
				break;
			}
				
			break;
			
			case KEY_ESC:
			done=1;
			break;
			
			case KEY_F5:
			done=1;
			exit=1;
			break;
		}
		
	
	}
	
	return exit;
}

void instructions (int *options) {
	border();
	
	if (!options[2]) {
		DrawStr(2,20,"You simply have to crack ",A_REPLACE);
		DrawStr(2,30,"the symbol combination.  ",A_REPLACE);
		DrawStr(2,40,"If you make a trial, the ",A_REPLACE);
		DrawStr(2,50,"program will give you    ",A_REPLACE);
		DrawStr(2,60,"hints.                   ",A_REPLACE);
		DrawStr(2,70,"A square means, that     ",A_REPLACE);
		DrawStr(2,80,"there's a symbol which...",A_REPLACE);		
	} else {
		DrawStr(2,20,"Das Ziel des Spieles ist ",A_REPLACE);
		DrawStr(2,30,"es, die Kombination von  ",A_REPLACE);
		DrawStr(2,40,"Symbolen zu knacken.     ",A_REPLACE);
		DrawStr(2,50,"Nach jedem Versuch gibt  ",A_REPLACE);
		DrawStr(2,60,"dir das Programm         ",A_REPLACE);
		DrawStr(2,70,"Hinweise.                ",A_REPLACE);
		DrawStr(2,80,"Ein Quadrat bedeutet, ...",A_REPLACE);				
	}

	anyKey(options);
	
	if (!options[2]) {
		DrawStr(2,20,"exists in the solution   ",A_REPLACE);
		DrawStr(2,30,"and which is placed      ",A_REPLACE);
		DrawStr(2,40,"right.                   ",A_REPLACE);
		DrawStr(2,50,"A line means, that       ",A_REPLACE);
		DrawStr(2,60,"there's a symbol which   ",A_REPLACE);
		DrawStr(2,70,"exists in the solution   ",A_REPLACE);
		DrawStr(2,80,"but which isn't placed...",A_REPLACE);
	} else {
		DrawStr(2,20,"dass ein Symbol in der   ",A_REPLACE);
		DrawStr(2,30,"Lsung vorkommt und zudem",A_REPLACE);
		DrawStr(2,40,"richtig plaziert ist.    ",A_REPLACE);
		DrawStr(2,50,"Ein Strichlein bedeutet, ",A_REPLACE);
		DrawStr(2,60,"dass ein Symbol zwar in  ",A_REPLACE);
		DrawStr(2,70,"der Lsung vorkommt, aber",A_REPLACE);
		DrawStr(2,80,"am falschen Ort...       ",A_REPLACE);
	}


	anyKey(options);

	if (!options[2]) {
		DrawStr(2,20,"right.                   ",A_REPLACE);
		DrawStr(2,30,"You win if all your      ",A_REPLACE);
		DrawStr(2,40,"symbols are placed right.",A_REPLACE);
		DrawStr(2,50,"                         ",A_REPLACE);
		DrawStr(2,60,"                         ",A_REPLACE);
		DrawStr(2,70,"                         ",A_REPLACE);
		DrawStr(2,80,"                         ",A_REPLACE);
	} else {
		DrawStr(2,20,"plaziert ist.            ",A_REPLACE);
		DrawStr(2,30,"Du gewinnst, wenn alle   ",A_REPLACE);
		DrawStr(2,40,"Symbole am richigen Ort  ",A_REPLACE);
		DrawStr(2,50,"sind.                    ",A_REPLACE);
		DrawStr(2,60,"                         ",A_REPLACE);
		DrawStr(2,70,"                         ",A_REPLACE);
		DrawStr(2,80,"                         ",A_REPLACE);
	}

	anyKey(options);
	
	if (!options[2]) {
		DrawStr(2,20,"Keys:                    ",A_REPLACE);
		DrawStr(2,30,"UP/DOWN:  chose symbol   ",A_REPLACE);
		DrawStr(2,40,"F2/CLEAR: change symbol  ",A_REPLACE);
		DrawStr(2,50,"F1/ENTER: try            ",A_REPLACE);
		DrawStr(2,60,"ESC:      escape         ",A_REPLACE);
		DrawStr(2,70,"F5:       exit program   ",A_REPLACE);
		DrawStr(2,80,"                         ",A_REPLACE);
	} else {
		DrawStr(2,20,"Tasten:                  ",A_REPLACE);
		DrawStr(2,30,"AUF/AB:   Symbol whlen  ",A_REPLACE);
		DrawStr(2,40,"F2/CLEAR: Symbol ndern  ",A_REPLACE);
		DrawStr(2,50,"F1/ENTER: versuchen      ",A_REPLACE);
		DrawStr(2,60,"ESC:      abbrechen      ",A_REPLACE);
		DrawStr(2,70,"F5:       Program beenden",A_REPLACE);
		DrawStr(2,80,"                         ",A_REPLACE);
	}

	anyKey(options);
	
}

void about (int *options) {
	border();
	
	if (!options[2]) {
		DrawStr(35,35," March 2004",A_NORMAL);
	} else {
		DrawStr(35,35," Mrz 2004",A_NORMAL);				
	}
	DrawStr(26,50,"Christian Helbling",A_NORMAL);
	DrawStr(40,65,"helch@gmx.ch",A_NORMAL);
	
	
	anyKey(options);
}

void anyKey (int *options) {
	FontSetSys(F_4x6);
	if (!options[2]) {
		DrawStr(45,90,"press any key ...",A_NORMAL);	
	} else {
		DrawStr(45,90,"druecke Taste ...",A_NORMAL);			
	}
	FontSetSys(F_6x8);
	
	ngetchx();	
}


void border(void) {

	ClrScr();

	FastDrawLine(LCD_MEM,0,99,159,99,A_NORMAL);
	FastDrawLine(LCD_MEM,0,0,0,99,A_NORMAL);
	FastDrawLine(LCD_MEM,0,0,159,0,A_NORMAL);
	FastDrawLine(LCD_MEM,159,0,159,99,A_NORMAL);

	FontSetSys(F_8x10);
	DrawStr(4,5,titel,A_NORMAL);
	FontSetSys(F_4x6);
	DrawStr(130,5,"  by ",A_NORMAL);
	DrawStr(130,11,"helch",A_NORMAL);
	FontSetSys(F_6x8);

	FastDrawLine(LCD_MEM,0,18,159,18,A_NORMAL);

}


int changeSymbol (int currentSymbol, int *options) {
	int a,key,newSymbol=currentSymbol;
	int offset = 27+(10-options[1])*6;
	
	for (a=0; a<options[1]; a++) {
		Sprite8_XOR(offset+2+12*a,22,8,symbols[a],LCD_MEM);		
	}
	
	Sprite16_XOR(offset+12*newSymbol,20,12,cursor1,LCD_MEM);		

	while (1) {
		
		key = ngetchx();
		
		switch (key) {

			case KEY_ESC:
			for (a=0; a<options[1]; a++) {
				Sprite8_XOR(offset+2+12*a,22,8,symbols[a],LCD_MEM);		
			}
			Sprite16_XOR(offset+12*newSymbol,20,12,cursor1,LCD_MEM);
			return currentSymbol;
			break;

			case KEY_ENTER:
			case KEY_CLEAR:
			case KEY_F2:
			for (a=0; a<options[1]; a++) {
				Sprite8_XOR(offset+2+12*a,22,8,symbols[a],LCD_MEM);		
			}
			Sprite16_XOR(offset+12*newSymbol,20,12,cursor1,LCD_MEM);
			return newSymbol;
			break;

			
			case KEY_RIGHT:
			Sprite16_XOR(offset+12*newSymbol,20,12,cursor1,LCD_MEM);			
			newSymbol++;
			if (newSymbol>options[1]-1) {
				newSymbol = 0;
			}
			Sprite16_XOR(offset+12*newSymbol,20,12,cursor1,LCD_MEM);		
			break;
			
			case KEY_LEFT:
			Sprite16_XOR(offset+12*newSymbol,20,12,cursor1,LCD_MEM);			
			newSymbol--;
			if (newSymbol<0) {
				newSymbol = options[1]-1;
			}
			Sprite16_XOR(offset+12*newSymbol,20,12,cursor1,LCD_MEM);		
			break;
		}
	
	}
}

void drawHint(int trials[NR_OF_HISTORY_ENTRIES][8],int cpx, int invisibleleft) {
	unsigned char *pxaddr;
	int a;
	
	for (a=0; a<4; a++) {
		if (trials[cpx][4+a]==2) {
			pxaddr = EXT_PIXADDR(LCD_MEM,25+12*(cpx-invisibleleft),86+3*a);
			EXT_SETPIX_AM(pxaddr,EXT_PIXMASK(25+12*(cpx-invisibleleft)));
			EXT_SETPIX_AM(pxaddr,(EXT_PIXMASK(25+12*(cpx-invisibleleft)))>>1);
			EXT_SETPIX_AM(pxaddr+30,EXT_PIXMASK(25+12*(cpx-invisibleleft)));
			EXT_SETPIX_AM(pxaddr+30,(EXT_PIXMASK(25+12*(cpx-invisibleleft)))>>1);
			
			
		} else if (trials[cpx][4+a]==1) {
			pxaddr = EXT_PIXADDR(LCD_MEM,25+12*(cpx-invisibleleft),86+3*a);
			EXT_SETPIX_AM(pxaddr,EXT_PIXMASK(25+12*(cpx-invisibleleft)));
			EXT_SETPIX_AM(pxaddr,(EXT_PIXMASK(25+12*(cpx-invisibleleft)))>>1);
			EXT_SETPIX_AM(pxaddr,(EXT_PIXMASK(25+12*(cpx-invisibleleft)))>>2);
			
		}
	}
	
}

int try(int trials[NR_OF_HISTORY_ENTRIES][8], int solution[NR_OF_SYMBOLS], int *attempt, int *cpx, int *cpy, int *invisibleleft, int *currentSymbol, int *options, HSC *highscores) {
	unsigned char *pxaddr=NULL;
	int a,b,temp,done=0;

	char winstr[25];

	
	//hints:
	
	for (a=0; a<4; a++) {
		if (trials[*attempt][a]==solution[a]) {
			trials[*attempt][4+a] = 2;
		} else if (trials[*attempt][a]==solution[0] || trials[*attempt][a]==solution[1] || trials[*attempt][a]==solution[2] || trials[*attempt][a]==solution[3] ) {
			trials[*attempt][4+a] = 1;
		} else {
			trials[*attempt][4+a] = 0;
		}
	}
	
	for (a=0; a<4; a++) {
		for (b=3; b>a; b--) {
			if (trials[*attempt][4+b]>trials[*attempt][3+b]) {
				temp=trials[*attempt][4+b];
				trials[*attempt][4+b]=trials[*attempt][3+b];
				trials[*attempt][3+b]=temp;
			}
		}
	}
	
	drawHint(trials, *attempt, *invisibleleft);
	
	
	//won?
	if (trials[*attempt][4]==2 && trials[*attempt][5]==2 && trials[*attempt][6]==2 && trials[*attempt][7]==2) {
		if (!options[2]) {
			sprintf(winstr,"You won!!! Trials: %d",*attempt+1);
		} else {
			sprintf(winstr,"Gewonnen! Versuche: %d",*attempt+1);
		}
		DrawStr(20,22,winstr,A_NORMAL);
		
		for (pxaddr=EXT_PIXADDR(LCD_MEM,3,36); pxaddr<EXT_PIXADDR(LCD_MEM,13,81); pxaddr+=29) {
			
			EXT_CLRPIX_AM(pxaddr,0x1F);
			pxaddr++;
			EXT_CLRPIX_AM(pxaddr,0xF8);
		}
		for (a=0;a<4;a++) {
			Sprite8_XOR(4,37+12*a,8,symbols[solution[a]],LCD_MEM);
		}
		ngetchx();
		
		//highscores:
		int level = options[0]*7+options[1]-4;
		if (*attempt+1 < (highscores[level].score)) {
			
			for (pxaddr=EXT_PIXADDR(LCD_MEM,15,34); pxaddr<EXT_PIXADDR(LCD_MEM,15,84); pxaddr+=12) {
				
				EXT_CLRPIX_AM(pxaddr,0x01);
				pxaddr++;
				for (a=0;a<17;a++) {
					EXT_CLRPIX_AM(pxaddr,0xFF);				
					pxaddr++;
				}
				EXT_CLRPIX_AM(pxaddr,0xFE);
			}
			for (pxaddr=EXT_PIXADDR(LCD_MEM,15,85); pxaddr<EXT_PIXADDR(LCD_MEM,15,99); pxaddr+=12) {
				
				EXT_CLRPIX_AM(pxaddr,0x01);
				pxaddr++;
				for (a=0;a<17;a++) {
					EXT_CLRPIX_AM(pxaddr,0xFF);
					pxaddr++;
				}
				EXT_CLRPIX_AM(pxaddr,0xFE);
			}
			
			if (!options[2]) {
				DrawStr(17,40,"You have a highscore!",A_NORMAL);
				DrawStr(17,50,"Please enter your name:",A_NORMAL);
				InputStr(highscores[level].name, 10, 17,60);

			} else {
				DrawStr(17,40,"Du hast eine Highscore!",A_NORMAL);
				DrawStr(17,50,"Gib bitte deinen",A_NORMAL);						
				DrawStr(17,60,"Namen ein:",A_NORMAL);						
				InputStr(highscores[level].name, 10, 81,60);

			}
			
			highscores[level].score = *attempt+1;
			saveFile(options, highscores);
			
			showHighscores(highscores, options);
		}
		
		return 1;
		done = 1;
	}
	
	
	
	Sprite16_XOR(21+12*(*cpx-*invisibleleft),35+12*(*cpy),12,cursor1,LCD_MEM);
	
	
	if (*attempt-*invisibleleft>10) {
		*cpx = *invisibleleft+10;
	}
	while (*attempt-*invisibleleft>10) {
		(*invisibleleft)++;
		(*cpx)++;
		//scroll 12px to the right:
		scrollLeft12();
		
		for (a=0; a<4; a++) {
			Sprite8_XOR(23+12*(*cpx-*invisibleleft),37+a*12,8,symbols[trials[*cpx][a]],LCD_MEM);
		}
			drawHint(trials,*cpx,*invisibleleft);
	}
	
	if (*attempt-*invisibleleft==10) {
		scrollLeft12();
		(*invisibleleft)++;
		(*cpx)++;
	} else {
	drawHint(trials,*cpx,*invisibleleft);
	}


	(*attempt)++;
	(*cpx)=*attempt;
	(*cpy)=0;
	
	
	for (a=0;a<4;a++) {
		trials[*attempt][a]=trials[*attempt-1][a];
		Sprite8_XOR(23+12*(*cpx-*invisibleleft),37+12*a,8,symbols[trials[*attempt][a]],LCD_MEM);
	}
		
	Sprite16_XOR(21+12*(*cpx-*invisibleleft),35+12*(*cpy),12,cursor1,LCD_MEM);			
	
	*currentSymbol = trials[*attempt][*cpy];
	
	return 0;
}

void scrollLeft12 (void) {
	int a;
	unsigned char *pxaddr;
	
	for (pxaddr = EXT_PIXADDR(LCD_MEM,23,37); pxaddr<EXT_PIXADDR(LCD_MEM,143,97); pxaddr+=13) {
		
		*pxaddr=(*(pxaddr+2)&0x10)>>4;
		
		pxaddr++;
		
		for (a=0; a<14; a++) {			
			*pxaddr=(*(pxaddr+1))<<4 | (*(pxaddr+2))>>4;
			pxaddr++;
		}
		*pxaddr=(*(pxaddr+1)&0x0E)<<4;

		pxaddr++;
		
		*pxaddr=0x00;
		pxaddr++;
		
		if (pxaddr == EXT_PIXADDR(LCD_MEM,153,80)) {
			pxaddr = EXT_PIXADDR(LCD_MEM,23,86)-13;
		}
	}		
}

void scrollRight12 (void) {
	int a;
	unsigned char *pxaddr;
		
	for (pxaddr = EXT_PIXADDR(LCD_MEM,151,97); pxaddr>EXT_PIXADDR(LCD_MEM,23,37); pxaddr-=13) {
							
		for (a=0; a<14; a++) {			
			*pxaddr=(*(pxaddr-1)&0xF0)>>4 | (*(pxaddr-2)&0x0F)<<4;
			pxaddr--;
		}
		
		*pxaddr=(*(pxaddr-2)&0x01)<<4 | (*(pxaddr-1))>>4;

		pxaddr--;
	
		*pxaddr=0x00;
		pxaddr--;
		
		*pxaddr = (*pxaddr & 0x01) ^ *pxaddr;
		pxaddr--;
	
	
		if (pxaddr == EXT_PIXADDR(LCD_MEM,23,86)-1) {
			pxaddr = EXT_PIXADDR(LCD_MEM,151,80)+13;
		}
	}		
}

// Main Function
void _main(void)
{
	int sel=1,key=0,done=0;
	int options[3];
	
	HSC *highscores = NULL;
	highscores = (HSC *)calloc((NR_OF_SYMBOLS-3)*2,sizeof(HSC));
	if (highscores == NULL) {
		DlgMessage("DMA Error","Too less free Memory!!!",BT_OK,BT_NONE);
		return;
	}


	int (*trials)[8] = NULL;
	trials = calloc(NR_OF_HISTORY_ENTRIES,sizeof(*trials));
	if (trials == NULL) {
		DlgMessage("DMA Error","Too less free Memory!!!",BT_OK,BT_NONE);
		free(highscores);
		return;
	}
	
	randomize();
	
	readFile(options, highscores);
	
	//menu:     
	while (!done) {
	
		border();
		
		if (!options[2]) {
			DrawStr(30,30,"Play",A_NORMAL);
			DrawStr(30,40,"Options",A_NORMAL);
			DrawStr(30,50,"High scores",A_NORMAL);
			DrawStr(30,60,"Instructions",A_NORMAL);
			DrawStr(30,70,"About",A_NORMAL);
			DrawStr(30,80,"Exit",A_NORMAL);
		} else {
			DrawStr(30,30,"Spielen",A_NORMAL);
			DrawStr(30,40,"Optionen",A_NORMAL);
			DrawStr(30,50,"Highscores",A_NORMAL);			
			DrawStr(30,60,"Anleitung",A_NORMAL);
			DrawStr(30,70,"ber",A_NORMAL);
			DrawStr(30,80,"Schliessen",A_NORMAL);		
		}
		
		Sprite8_XOR(20,20+sel*10,8,flash,LCD_MEM);
		while ((key = select (20, 30, 10, 6, &sel))==KEY_LEFT);
		
		switch (key) {
			
			case KEY_RIGHT:
			case KEY_ENTER:
			switch (sel) {
		
				case 1:								//play
				case KEY_ENTER:					
				done = newGame(trials, options, highscores);
				break; 
		
				case 2:								//options
				done = editOptions(options, highscores);
				break; 
				
				case 3:								//highscores
				showHighscores(highscores,options);
				break;
		
				case 4:								//instructions
				instructions(options);
				break; 
		
				case 5:								//about
				about(options);
				break; 
		
				case 6:								//exit
				done = 1;
				break; 
			}
			break;
			
			case KEY_ESC:
			case KEY_F5:
			done=1;
			break;
			
			
		}

	} 

	free(highscores);
	free(trials);
}
